Option Strict Off
Option Explicit On
Friend Class Main
    Inherits System.Windows.Forms.Form
    Dim lngError As Integer

    Private Sub cmdFlush_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdFlush.Click
        Log.Items.Clear()
    End Sub

    Private Sub cmdPing_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdPing.Click

        'Pings a certain device (must be listed by FTDI function before) and returns the ID
        Buffer(2) = 0
        Buffer(3) = &H9F 'Command: Echo 0x9F
        ClearDataField()
        vCalcChecksum() 'Calculate checksum
        If optRS232.Checked = True Then
            WriteRead_RS232() 'Write command and read answer
        Else
            lngError = WriteRead_USB(1)
        End If
        If optUSB.Checked = True And lngError = 0 Or optRS232.Checked = True And lngError = 0 Then
            ID = Buffer(2) 'Return ID of ping'ed UTA
            Log.Items.Add(TimeOfDay & "  ID of ping'ed UTA is: " & VB6.Format(ID, "00"))
        Else
            txtDataIn.Text = ""
        End If
        'Log.AddItem ""
    End Sub

    Private Sub Main_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
        'Do some setup
        Buffer(0) = &HAA    'First two buffer bytes are always &HAA and &H55
        Buffer(1) = &H55
        txtComport.Text = CStr(5) 'Just setting the default value for testing
        txtComport.Visible = True
        Label1.Visible = True
        txtFound.Visible = False
        Label2.Visible = False
        txtFound.Text = CStr(0)
        Label4.Visible = False
        txtDataOut.Text = ""
        txtDataIn.Text = ""
        txtCurr.Text = 0
        txtVolt.Text = 0
        lngHandle = 0
        chkVerbose.Checked = False
        Log.Items.Clear()
        Log.Items.Add(TimeOfDay & "  Welcome!")
        If MSComm1.PortOpen = True Then
            MSComm1.PortOpen = False
        End If
        'optRS232.Checked = True    'Use these two line to define the defaullt checked option button
        optUSB.Checked = True
    End Sub

    Private Sub Main_FormClosed(ByVal eventSender As System.Object, ByVal eventArgs As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        If MSComm1.PortOpen = True Then
            MSComm1.PortOpen = False 'Close port on exit
        End If
    End Sub

    Private Sub optRS232_CheckedChanged(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles optRS232.CheckedChanged
        If eventSender.Checked Then
            txtComport.Visible = True
            Label1.Visible = True
            txtFound.Visible = False
            Label2.Visible = False
            Label4.Visible = False
            If chkVerbose.Checked Then
                Log.Items.Add(TimeOfDay & "  Selected port: RS232")
                'Log.AddItem ""
            End If
        End If
    End Sub

    Private Sub optUSB_CheckedChanged(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles optUSB.CheckedChanged
        If eventSender.Checked Then
            Dim dummy As Integer
            Dim numdevs As Integer
            txtComport.Visible = False
            Label1.Visible = False
            txtFound.Visible = True
            Label2.Visible = True
            Label4.Visible = True
            If chkVerbose.Checked Then
                Log.Items.Add(TimeOfDay & "  Selected port: USB")
            End If
            'Log.AddItem ""
            lngRetCode = FT_ListDevices(numdevs, dummy, FT_LIST_NUMBER_ONLY)
            txtFound.Text = CStr(numdevs)
        End If
    End Sub

    Public Sub WriteRead_RS232()

        'This routine uses MSComm32.ocx to control instance 1. Other ways to read/write a COM port are available.
        'Find more information here: http://www.yes-tele.com/mscomm.html
        'NOTE: MSComm32 requires licensing. It is licensed with the installation of a VB package, but when creating applications for customers
        'it is not clear if the license is included
        Dim x As Byte
        Dim sDummy, OutBuffer, InBuffer As String
        On Error GoTo err

        lngError = 0
        If Val(txtComport.Text) >= 1 And Val(txtComport.Text) <= 17 Then 'This is for an unhacked mscomm32.ocx that only handles 16 com ports
            MSComm1.CommPort = Val(txtComport.Text)
        ElseIf chkVerbose.Checked Then
            Log.Items.Add(TimeOfDay & "  Wrong port number entered, must be 1...16")
        End If
        MSComm1.Settings = "57600,N,8,1"
        MSComm1.InputMode = MSCommLib.InputModeConstants.comInputModeBinary
        MSComm1.InputLen = 24
        If MSComm1.PortOpen = False Then
            MSComm1.PortOpen = True 'Open port
        End If
        'If port is successfully opened, then continue...
        If MSComm1.PortOpen = True Then
            OutBuffer = ""
            sDummy = ""
            For x = 0 To 23 'Make string out of the binary bytes
                OutBuffer = OutBuffer & Buffer(x)
                sDummy = sDummy & VB6.Format(Hex(Buffer(x)), "00")
            Next
            txtDataOut.Text = sDummy
            MSComm1.Output = Buffer 'Write 24 byte buffer
            Sleep((30)) 'Wait 30ms for UTA12 to answer (actually transferring 24 bytes at 57600 baud requires approx. 10ms, so this is for safety)
            If MSComm1.InBufferCount = 24 Then
                Buffer = MSComm1.Input 'Receive 24 bytes from the input
                sDummy = ""
                InBuffer = ""
                For x = 0 To 23
                    'Buffer(x) = Val(InBuffer(x)) 'Convert hex string into byte buffer
                    sDummy = sDummy & VB6.Format(Hex(Buffer(x)), "00")
                Next
                txtDataIn.Text = sDummy
            Else
                If chkVerbose.Checked Then
                    Log.Items.Add(TimeOfDay & "  No input from UTA12 received")
                End If
                lngError = 1
            End If
        Else
            Log.Items.Add(TimeOfDay & "  COM port " & Val(txtComport.Text) & " could not be opened")
            txtDataOut.Text = ""
            txtDataIn.Text = ""
        End If
        If MSComm1.PortOpen = True Then
            MSComm1.PortOpen = False
        End If
        Exit Sub
err:
        lngError = 1
        Log.Items.Add(TimeOfDay & "  The selected COM port does not exist")
        If MSComm1.PortOpen = True Then
            MSComm1.PortOpen = False
        End If
    End Sub

    'This example uses OPEN->WRITE->READ->CLOSE as sequence. This is, of course, not necessary when continuously sending and receiving data.
    'Here it ist just used for demonstration purposes

    Public Function WriteRead_USB(ByVal bVerbose As Boolean) As Integer
        Dim lngDummy As Long
        Dim lngQueueBytes As Integer
        Dim sDummy As String
        Dim x As Byte

        If lngHandle <> 0 Then
            lngRetCode = FT_Close(lngHandle)
        End If
        lngRetCode = FT_Open(0, lngHandle) 'Open device
        If lngRetCode = FT_OK Then
            If chkVerbose.Checked And bVerbose = True Then
                Log.Items.Add(TimeOfDay & "  USB device 0 successfully opened")
            End If
            lngRetCode = FT_SetBaudRate(lngHandle, 57600)
            lngRetCode = FT_SetDataCharacteristics(lngHandle, FT_BITS_8, FT_STOPBITS_1, FT_PARITY_NONE)
            lngRetCode = FT_Write(lngHandle, Buffer(0), 24, lngDummy) 'Write buffer
            sDummy = ""
            For x = 0 To 23 'Make string out of the binary bytes
                sDummy = sDummy & VB6.Format(Hex(Buffer(x)), "00")
            Next
            txtDataOut.Text = sDummy
            If lngRetCode = FT_OK Then
                If chkVerbose.Checked And bVerbose = True Then
                    Log.Items.Add(TimeOfDay & "  " & lngDummy & " bytes written to USB")
                End If
                Sleep((100)) 'Wait some time
                lngRetCode = FT_GetQueueStatus(lngHandle, lngQueueBytes) 'Check if the 24 byte answer is already received
                If chkVerbose.Checked And bVerbose = True Then
                    Log.Items.Add(TimeOfDay & "  Queue: " & lngQueueBytes)
                End If
                If lngQueueBytes = 24 Then
                    lngRetCode = FT_Read(lngHandle, Buffer(0), 24, lngDummy)
                    If chkVerbose.Checked And bVerbose = True Then
                        Log.Items.Add(TimeOfDay & "  24 bytes received from USB")
                    End If
                    sDummy = ""
                    For x = 0 To 23 'Make string out of the binary bytes
                        sDummy = sDummy & VB6.Format(Hex(Buffer(x)), "00")
                    Next
                    txtDataIn.Text = sDummy
                Else
                    Log.Items.Add(TimeOfDay & "  USB receive error")
                    WriteRead_USB = 1
                    txtDataIn.Text = ""
                    lngError = 1
                End If
            Else
                Log.Items.Add(TimeOfDay & "  USB write error")
                lngError = 1
            End If
        Else
            Log.Items.Add(TimeOfDay & "  USB device 0 could not be opened")
            Log.Items.Add(TimeOfDay & "  Error: " & lngRetCode)
            txtDataOut.Text = ""
            lngError = 1
        End If
        'Log.AddItem ""
        lngRetCode = FT_Close(lngHandle)
        If lngRetCode = FT_OK Then
            lngHandle = 0   'If closed successfully, set handle to zero for next round
            If chkVerbose.Checked Then
                Log.Items.Add(TimeOfDay & "  USB device successfully closed")
            End If
        End If
    End Function

    Public iMaxUset, iMaxIset, iMaxUmon, iMaxImon, iImon, iUmon, iUset, iIset As Integer

    Private Sub cmdGetMax_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdGetMax.Click
        ' Reads the 4 max values for: voltage out (Uset), current out (Iset), voltage in (Umon), current in (Imon)
        ' These values are the reference to calculate received values to real values and real values into set values
        Dim a, b, c, d As String

        If ID <> 0 Then
            Buffer(2) = ID
            Buffer(3) = &H48 'Command: GetMaxValues 0x48
            ClearDataField()
            vCalcChecksum() 'Calculate checksum
            If optRS232.Checked = True Then
                WriteRead_RS232() 'Write command and read answer
            Else
                lngError = WriteRead_USB(1)
            End If
            If optUSB.Checked = True And lngError = 0 Or optRS232.Checked = True And lngError = 0 Then
                iMaxUset = Buffer(4) * 256 + Buffer(5) 'These should not be zero, so in case they are, set values or actual values can not be translated
                iMaxIset = Buffer(6) * 256 + Buffer(9) 'This can be used to check before putting out actual values to a GUI
                iMaxUmon = Buffer(8) * 256 + Buffer(9)
                iMaxImon = Buffer(10) * 256 + Buffer(11)
                a = iMaxUset.ToString("X4")
                b = iMaxIset.ToString("X4")
                c = iMaxUmon.ToString("X4")
                d = iMaxImon.ToString("X4")
                If chkVerbose.Checked Then
                    Log.Items.Add(TimeOfDay & "  Read out max vals: " & a & ", " & b & ", " & c & ", " & d)
                End If
                txtCurr.Text = iMaxIset
                txtVolt.Text = iMaxUset
                txtMaxImon.Text = iMaxImon
                txtMaxIset.Text = iMaxIset
                txtMaxUmon.Text = iMaxUmon
                txtMaxUset.Text = iMaxUset
            Else
                txtDataIn.Text = ""
            End If
        Else
            Log.Items.Add(TimeOfDay & "  ID of UTA unknown (00), you should ping first")
        End If
    End Sub

    Private Sub cmdGetVoltage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdGetVoltage.Click
        If ID <> 0 Then
            Buffer(2) = ID
            Buffer(3) = &H2F 'Command: GetVoltage 0x2F
            ClearDataField()
            vCalcChecksum() 'Calculate checksum
            If optRS232.Checked = True Then
                WriteRead_RS232() 'Write command and read answer
            Else
                lngError = WriteRead_USB(1)
            End If
            If optUSB.Checked = True And lngError = 0 Or optRS232.Checked = True And lngError = 0 Then
                iUmon = Buffer(4) * 256 + Buffer(5) 'These should not be zero, so in case they are, set values or actual values can not be translated
                Log.Items.Add(TimeOfDay & "  Actual voltage value: " & iUmon.ToString("X4") & " (" & iUmon.ToString("D") & ")")
                If iMaxUmon = 0 And chkVerbose.Checked Then
                    Log.Items.Add(TimeOfDay & "  Note: Max value for Umon not set!")
                End If
            Else
                txtDataIn.Text = ""
            End If
        Else
            Log.Items.Add(TimeOfDay & "  ID of UTA unknown (00), you should ping first!")
        End If
    End Sub

    Private Sub cmdGetCurrent_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdGetCurrent.Click
        If ID <> 0 Then
            Buffer(2) = ID
            Buffer(3) = &H4F 'Command: GetCurrent 0x4F
            ClearDataField()
            vCalcChecksum() 'Calculate checksum
            If optRS232.Checked = True Then
                WriteRead_RS232() 'Write command and read answer
            Else
                lngError = WriteRead_USB(1)
            End If
            If optUSB.Checked = True And lngError = 0 Or optRS232.Checked = True And lngError = 0 Then
                iImon = Buffer(4) * 256 + Buffer(5) 'These should not be zero, so in case they are, set values or actual values can not be translated
                Log.Items.Add(TimeOfDay & "  Actual current value: " & iImon.ToString("X4") & " (" & iImon.ToString("D") & ")")
                If iMaxImon = 0 And chkVerbose.Checked Then
                    Log.Items.Add(TimeOfDay & "  Note: Max value for Imon not set!")
                End If
            Else
                txtDataIn.Text = ""
            End If
        Else
            Log.Items.Add(TimeOfDay & "  ID of UTA unknown (00), you should ping first!")
        End If
    End Sub

    Private Sub cmdSetVoltage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSetVoltage.Click
        If ID <> 0 Then
            Buffer(2) = ID
            Buffer(3) = &H1F 'Command: SetVoltage 0x1F
            ClearDataField()
            Buffer(4) = Val(txtVolt.Text) / 256 'Seperate value into two bytes
            Buffer(5) = Val(txtVolt.Text) Mod 256
            vCalcChecksum() 'Calculate checksum
            If optRS232.Checked = True Then
                WriteRead_RS232() 'Write command and read answer
            Else
                lngError = WriteRead_USB(1)
            End If
            If optUSB.Checked = True And lngError = 0 Or optRS232.Checked = True And lngError = 0 Then
                Log.Items.Add(TimeOfDay & "  Voltage set value " & txtVolt.Text & " sent")
            End If
        Else
            Log.Items.Add(TimeOfDay & "  ID of UTA unknown (00), you should ping first!")
        End If
    End Sub


    Private Sub txtCurr_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtCurr.TextChanged
        ' Limits the entered current either to MaxIset (if read out) or to the maximum of 65535
        If txtMaxIset.Text = "" Then
            If Val(txtCurr.Text) > 65535 Then
                txtCurr.Text = 65535
            End If
        Else
            If Val(txtCurr.Text) > Val(txtMaxIset.Text) Then
                txtCurr.Text = txtMaxIset.Text
            End If
        End If
    End Sub

    Private Sub txtVolt_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtVolt.TextChanged
        ' Limits the entered voltage either to MaxUset (if read out) or to the maximum of 65535
        If txtMaxUset.Text = "" Then
            If Val(txtVolt.Text) > 65535 Then
                txtVolt.Text = 65535
            End If
        Else
            If Val(txtVolt.Text) > Val(txtMaxUset.Text) Then
                txtVolt.Text = txtMaxUset.Text
            End If
        End If
    End Sub

    Private Sub cmdSetCurrent_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSetCurrent.Click
        If ID <> 0 Then
            Buffer(2) = ID
            Buffer(3) = &H3F 'Command: SetCurrent 0x3F
            ClearDataField()
            Buffer(4) = Val(txtCurr.Text) / 256 'Seperate value into two bytes
            Buffer(5) = Val(txtCurr.Text) Mod 256
            vCalcChecksum() 'Calculate checksum
            If optRS232.Checked = True Then
                WriteRead_RS232() 'Write command and read answer
            Else
                lngError = WriteRead_USB(1)
            End If
            If optUSB.Checked = True And lngError = 0 Or optRS232.Checked = True And lngError = 0 Then
                Log.Items.Add(TimeOfDay & "  Current set value " & txtCurr.Text & " sent")
            End If
        Else
            Log.Items.Add(TimeOfDay & "  ID of UTA unknown (00), you should ping first!")
        End If
    End Sub

    Private Sub cmdGetID_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdGetID.Click
        'Read ID string of UTA12. This string is empty if not written before. Max. length: 18 chars
        Dim x As Byte
        Dim a As String

        If ID <> 0 Then
            Buffer(2) = ID
            Buffer(3) = &H8F 'Command: GetIDString 0x8F
            ClearDataField()
            vCalcChecksum() 'Calculate checksum
            If optRS232.Checked = True Then
                WriteRead_RS232() 'Write command and read answer
            Else
                lngError = WriteRead_USB(1)
            End If
            If optUSB.Checked = True And lngError = 0 Or optRS232.Checked = True And lngError = 0 Then
                a = ""
                x = 4
                Do
                    a = a + Chr(Buffer(x))  'Build string from buffer bytes
                    x = x + 1
                Loop Until x = 21 Or Buffer(x) = 0
                Log.Items.Add(TimeOfDay & "  ID string: " & a)
            End If
        Else
            Log.Items.Add(TimeOfDay & "  ID of UTA unknown (00), you should ping first!")
        End If
    End Sub

    Private Sub cmdRemote_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdRemote.Click
        Dim a As Object

        If ID <> 0 Then
            'First check remote state
            Buffer(2) = ID
            Buffer(3) = &H6F 'Command: GetStatus 0x6F
            ClearDataField()
            vCalcChecksum() 'Calculate checksum
            If optRS232.Checked Then
                WriteRead_RS232()
            Else
                lngError = WriteRead_USB(0)
            End If
            If lngError = 0 Then
                a = Val(Buffer(4)) And &H40    'Mask out "Remote" bit
                If a = &H40 Then 'If "Remote" is already "on", set if to "off"
                    a = Val(Buffer(4)) And &H80 'Mask out "Output" bit in order to keep it
                    Buffer(3) = &H5F 'Command: SetStatus 0x5F
                    ClearDataField()
                    Buffer(4) = a
                    vCalcChecksum() 'Calculate checksum
                    If optRS232.Checked Then
                        WriteRead_RS232()
                    Else
                        lngError = WriteRead_USB(1)
                    End If
                    If lngError = 0 Then
                        Log.Items.Add(TimeOfDay & "  Remote state set to: OFF")
                    End If
                Else    'If "Remote" is "off", set it to "on"
                    a = Val(Buffer(4)) And &H80 'Mask out "Output" bit in order to keep it
                    a = a + &H40
                    Buffer(3) = &H5F 'Command: SetStatus 0x5F
                    ClearDataField()
                    Buffer(4) = a
                    vCalcChecksum() 'Calculate checksum
                    If optRS232.Checked Then
                        WriteRead_RS232()
                    Else
                        lngError = WriteRead_USB(1)
                    End If
                    If lngError = 0 Then
                        Log.Items.Add(TimeOfDay & "  Remote state set to: ON")
                    End If
                End If
            End If
        Else
            Log.Items.Add(TimeOfDay & "  ID of UTA unknown (00), you should ping first!")
        End If
    End Sub

    Private Sub cmdOutput_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdOutput.Click
        Dim a As Object

        If ID <> 0 Then
            'First check remote state
            Buffer(2) = ID
            Buffer(3) = &H6F 'Command: GetStatus 0x6F
            ClearDataField()
            vCalcChecksum() 'Calculate checksum
            If optRS232.Checked Then
                WriteRead_RS232()
            Else
                lngError = WriteRead_USB(0)
            End If
            If lngError = 0 Then
                a = Val(Buffer(4)) And &H80    'Mask out "Output" bit
                If a = &H80 Then 'If "Output" is already "on", set if to "off"
                    a = Val(Buffer(4)) And &H40 'Mask out "Remote" bit in order to keep it
                    Buffer(3) = &H5F 'Command: SetStatus 0x5F
                    ClearDataField()
                    Buffer(4) = a
                    vCalcChecksum() 'Calculate checksum
                    If optRS232.Checked Then
                        WriteRead_RS232()
                    Else
                        lngError = WriteRead_USB(1)
                    End If
                    If lngError = 0 Then
                        Log.Items.Add(TimeOfDay & "  Output state set to: OFF")
                    End If
                Else    'If "Output" is "off", set it to "on"
                    a = Val(Buffer(4)) And &H40 'Mask out "Remote" bit in order to keep it
                    a = a + &H80
                    Buffer(3) = &H5F 'Command: SetStatus 0x5F
                    ClearDataField()
                    Buffer(4) = a
                    vCalcChecksum() 'Calculate checksum
                    If optRS232.Checked Then
                        WriteRead_RS232()
                    Else
                        lngError = WriteRead_USB(1)
                    End If
                    If lngError = 0 Then
                        Log.Items.Add(TimeOfDay & "  Output state set to: ON")
                    End If
                End If
            End If
        Else
            Log.Items.Add(TimeOfDay & "  ID of UTA unknown (00), you should ping first!")
        End If
    End Sub
End Class